use serenity::{
    async_trait,
    builder::CreateInteractionResponseData,
    model::prelude::{
        command::{Command, CommandOptionType},
        interaction::application_command::ApplicationCommandInteraction,
    },
    prelude::Context,
};

use crate::database::Client as DatabaseClient;

pub struct BotCommandOption {
    pub name: String,
    pub description: String,
    pub kind: CommandOptionType,
    pub required: bool,
}

#[async_trait]
pub trait BotCommand {
    fn new(context: ApplicationCommandInteraction) -> Self;
    fn name() -> String;
    fn description() -> String;
    fn options_list() -> Vec<BotCommandOption>;
    async fn run(&self, response: &mut CreateInteractionResponseData, database: &DatabaseClient);

    async fn register(context: &Context) {
        match Command::create_global_application_command(context, |command| {
            let mut new_command = command.name(Self::name()).description(Self::description());

            for opt in Self::options_list() {
                new_command = new_command.create_option(|option| {
                    option
                        .name(opt.name)
                        .description(opt.description)
                        .kind(opt.kind)
                        .required(opt.required)
                });
            }
            new_command
        })
        .await
        {
            Ok(_) => println!("Successfully registered the {} command", Self::name()),
            Err(_) => panic!("Failed to register the {} command", Self::name()),
        };
    }
}
